#  Copyright 2007 Steve Lee. All rights reserved.
#
#  This file is part of ClipTalk and is distributable under the BSD license.
#  See the file LICENSE.txt provided with this software.

import wx
import wx.lib.hyperlink as hl

import win32api
import win32gui
import win32con
import win32clipboard
import win32com.client 

# check if running for an installer such as py2exe
in_py2exe = win32com.client.gencache.is_readonly
if in_py2exe:

    #allow gencache to create the cached wrapper objects
    win32com.client.gencache.is_readonly = False
 
    # under p2exe the call in gencache to __init__() does not happen
    # so we use Rebuild() to force the creation of the gen_py folder
    win32com.client.gencache.Rebuild()

    # NB You must ensure that the python...\win32com.client.gen_py dir does not 
    # exist to allow creation of the cache in %temp%

from win32com.client.gencache import EnsureDispatch
from win32com.client import constants

from optparse import OptionParser

# based on code in the Active State Cookbook 
# http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/355593

class ThePanel(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent, -1)
        self.SetFont(wx.Font(12, wx.SWISS, wx.NORMAL, wx.BOLD, False))
        self.SetBackgroundColour("#ffffcc")
        self.SetForegroundColour("#990000")
 
         # some text and a link back to base
        sizer = wx.BoxSizer(wx.VERTICAL)
        self.SetSizer(sizer) 
        text = """ClipTalk automatically speaks text as it is copied to the clipboard.
        
Open Source Assistive Technology Software such as ClipTalk may be freely copied or given to others.\
 Software developers can use the code in order to add new features or create new programs."""        
        text1 = wx.StaticText(self, -1, text)
        sizer.Add((0,10))
        w,h = parent.GetClientSizeTuple()
        text1.Wrap(w - 12)
        sizer.Add(text1, 0, wx.ALL, 6)
            
        self._hyper1 = hl.HyperLinkCtrl(self, wx.ID_ANY, "Find out more at fullmeasure.co.uk",
                                        URL="http://fullmeasure.co.uk/")
        sizer.Add(self._hyper1, 0, wx.ALL, 6)


class TestFrame (wx.Frame):
    def __init__ (self):
        wx.Frame.__init__ (self, None, title="ClipTalk - Speaks Clipboard text" )

        self.first   = True
        self.nextWnd = None

        # Get native window handle of this wxWidget Frame.
        self.hwnd    = self.GetHandle ()

        # Set the WndProc to our function.
        self.oldWndProc = win32gui.SetWindowLong (self.hwnd,
                                                  win32con.GWL_WNDPROC,
                                                  self.MyWndProc)
        try:
            self.nextWnd = win32clipboard.SetClipboardViewer (self.hwnd)
        except win32api.error:
            if win32api.GetLastError () == 0:
                # there is no other window in chain
                pass
            else:
                raise
        try:
            self.v = EnsureDispatch("Sapi.SpVoice", bForDemand=0)
        except:
            from win32ui import MessageBox
            from win32con import MB_OK, MB_ICONERROR
            MessageBox('Unable to use Speech.\n'
                        'Please check that Microsoft SAPI 5.1 speech is available.',
                        'ClipTalk', MB_OK | MB_ICONERROR)
            raise SystemExit            

        # parse the command line options
        self.parseOptions()

        # set the speech flags to async and purge
        self.sflags |= constants.SVSFlagsAsync
        self.sflags |= constants.SVSFPurgeBeforeSpeak

        panel = ThePanel(self)
 
        if self.announce:
            self.say('ClipTalk is now monitoring the clipboard')

    def MyWndProc (self, hWnd, msg, wParam, lParam):
        if msg == win32con.WM_CHANGECBCHAIN:
            self.OnChangeCBChain (msg, wParam, lParam)
        elif msg == win32con.WM_DRAWCLIPBOARD:
            self.OnDrawClipboard (msg, wParam, lParam)

        # Restore the old WndProc. Notice the use of win32api
        # instead of win32gui here. This is to avoid an error due to
        # not passing a callable object.
        if msg == win32con.WM_DESTROY:
            if self.nextWnd:
               win32clipboard.ChangeClipboardChain (self.hwnd, self.nextWnd)
            else:
               win32clipboard.ChangeClipboardChain (self.hwnd, 0)

            win32api.SetWindowLong (self.hwnd,
                                    win32con.GWL_WNDPROC,
                                    self.oldWndProc)

        if msg == win32con.WM_CLOSE:
            if self.announce:
                #remove the async flag so we hear it before we quit.
                self.sflags &= ~constants.SVSFlagsAsync
                self.say('ClipTalk is closing')

        # Pass all messages (in this case, yours may be different) on
        # to the original WndProc
        return win32gui.CallWindowProc (self.oldWndProc,
                                        hWnd, msg, wParam, lParam)

    def OnChangeCBChain (self, msg, wParam, lParam):
        if self.nextWnd == wParam:
           # repair the chain
           self.nextWnd = lParam
        if self.nextWnd:
           # pass the message to the next window in chain
           win32api.SendMessage (self.nextWnd, msg, wParam, lParam)

    def OnDrawClipboard (self, msg, wParam, lParam):
        if self.first:
           self.first = False
        else:
            self.say(self.GetTextFromClipboard())
#           print "clipboard content changed"
        if self.nextWnd:
           # pass the message to the next window in chain
           win32api.SendMessage (self.nextWnd, msg, wParam, lParam)
           
    def say(self, txt):
        #process newline pauses
        if self.pauses:
            s = "<silence msec='%d'/>" % self.pauses
            txt = txt.replace('\n', s)
            
        #speak    
        try:
            self.v.Speak( txt, self.sflags )
        except:
            pass    # doesn't seem to be much useful info

    def GetTextFromClipboard(self):
        """
        """
        clipboard = wx.Clipboard()
        try:
            if clipboard.Open():
                if clipboard.IsSupported(wx.DataFormat(wx.DF_TEXT)):
                    data = wx.TextDataObject()
                    clipboard.GetData(data)
                    s = data.GetText()
                    #self.tc.AppendText("Clip content:\n%s\n\n" % s )
                    clipboard.Close()
                    return s
        except:
            try:
                clipboard.Close()
            except:
                pass
        return ""

    def parseOptions(self):
        '''
        Parses the command line arguments based on the definition of the expected
        arguments provided here.
        '''
        # create a command line option parser
        op = OptionParser(version="%prog 0.1.0",
                        description="Automatically speak text copied to the clipboard.")

        # add all the command line options to the parser
        op.set_defaults(process_markup="no", persist_markup=False, announce=True, 
                        pron_punc=False, pauses=0)
        op.add_option('-m', '--markup',  action='store', dest='process_markup',
                    choices=['no', 'yes', 'auto'], metavar='MODE', 
                    help="process SAPI XML Synthesis markup: no, yes, auto [default: %default]" )
        op.add_option('-g', '--global-markup', action='store_true', dest='persist_markup',
                    help="persist global markup between utterances" )
        op.add_option("-p", "--punctuation", action="store_true", dest="pron_punc",
                    help='pronounce punctuation as words')
        op.add_option("-q", "--quiet", action="store_false", dest="announce",
                    help='no announcements at startup and exit')
        op.add_option("-l", "--line-pauses", action="store", type ='int', dest="pauses",
                        metavar="PAUSE",
                    help='automatic pauses of PAUSE ms for newlines (implies -m yes)')

        options, args = op.parse_args()
        
        # set the speech flags
        self.sflags = constants.SVSFDefault
        if options.process_markup == 'no':
            self.sflags |= constants.SVSFIsNotXML
        elif options.process_markup == 'yes' \
            or options.pauses:  # pauses need XML
            self.sflags |= constants.SVSFIsXML
        if options.persist_markup == 'yes':
            self.sflags |= constants.SVSFPersistXML
        if options.pron_punc:
            self.sflags |= constants.SVSFNLPSpeakPunc

        self.announce = options.announce;
        self.pauses = options.pauses;

def main():
    if in_py2exe:
        import cStringIO, sys
        strioout = cStringIO.StringIO()
        strioerr = cStringIO.StringIO()
        saveout = sys.stdout
        saveerr = sys.stderr
        sys.stdout = strioout
        sys.stderr = strioerr
        
        def print_std(strio, title):
            if len(strio.getvalue()):
                from win32ui import MessageBox
                MessageBox(strio.getvalue(), title)

        # restore, probably not necessary
        def restore(out, err):    
            sys.stdout = out
            sys.stdout = err

        import atexit
        atexit.register(print_std, strioout, "ClipTalk Output")
        atexit.register(print_std, strioerr, "ClipTalk Errors occured")
        atexit.register(restore, saveout, saveerr)

    app   = wx.PySimpleApp()
    frame = TestFrame()
    frame.Show()

    app.MainLoop()

        
if __name__ == '__main__':
  main()
